"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.shade = exports.tint = exports.argb = exports.contrast = exports.greyscale = exports.mix = exports.spin = exports.fade = exports.fadeout = exports.fadein = exports.darken = exports.lighten = exports.desaturate = exports.saturate = exports.luminance = exports.luma = exports.alpha = exports.blue = exports.green = exports.red = exports.hsvvalue = exports.hsvsaturation = exports.hsvhue = exports.lightness = exports.saturation = exports.hue = exports.hsva = exports.hsv = exports.hsla = exports.hsl = exports.rgba = exports.rgb = void 0;

var _VariableNode = _interopRequireDefault(require("../tree/VariableNode"));

var _convert = require("../tree/convert");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var parse = function parse(color) {
  return new _VariableNode["default"]("require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toRgbString()"));
};

var rgb = function rgb(r, g, b) {
  return rgba(r, g, b, 1);
};

exports.rgb = rgb;

var rgba = function rgba(r, g, b, a) {
  if (g) {
    if (a == 1) {
      return new _VariableNode["default"]("require(\"tinycolor2\")({ r: ".concat((0, _convert.convertNode)(r), ", g: ").concat((0, _convert.convertNode)(g), ", b: ").concat((0, _convert.convertNode)(b), " }).toHexString()"));
    } else {
      return new _VariableNode["default"]("require(\"tinycolor2\")({ r: ".concat((0, _convert.convertNode)(r), ", g: ").concat((0, _convert.convertNode)(g), ", b: ").concat((0, _convert.convertNode)(b), ", a: ").concat((0, _convert.convertNode)(a), " }).toRgbString()"));
    }
  } else {
    return parse(r);
  }
};

exports.rgba = rgba;

var hsl = function hsl(h, s, l) {
  return hsla(h, s, l, 1);
};

exports.hsl = hsl;

var hsla = function hsla(h, s, l, a) {
  if (s) {
    if (a == 1) {
      return new _VariableNode["default"]("require(\"tinycolor2\")({ h: ".concat((0, _convert.convertNode)(h), ", s: ").concat((0, _convert.convertNode)(s), ", l: ").concat((0, _convert.convertNode)(l), " }).toHexString()"));
    } else {
      return new _VariableNode["default"]("require(\"tinycolor2\")({ h: ".concat((0, _convert.convertNode)(h), ", s: ").concat((0, _convert.convertNode)(s), ", l: ").concat((0, _convert.convertNode)(l), ", a: ").concat((0, _convert.convertNode)(a), " }).toRgbString()"));
    }
  } else {
    return parse(h);
  }
};

exports.hsla = hsla;

var hsv = function hsv(h, s, v) {
  return hsva(h, s, v, 1);
};

exports.hsv = hsv;

var hsva = function hsva(h, s, v, a) {
  if (s) {
    if (a == 1) {
      return new _VariableNode["default"]("require(\"tinycolor2\")({ h: ".concat((0, _convert.convertNode)(h), ", s: ").concat((0, _convert.convertNode)(s), ", v: ").concat((0, _convert.convertNode)(v), " }).toHexString()"));
    } else {
      return new _VariableNode["default"]("require(\"tinycolor2\")({ h: ".concat((0, _convert.convertNode)(h), ", s: ").concat((0, _convert.convertNode)(s), ", v: ").concat((0, _convert.convertNode)(v), ", a: ").concat((0, _convert.convertNode)(a), " }).toRgbString()"));
    }
  } else {
    return parse(h);
  }
};

exports.hsva = hsva;

var hue = function hue(color) {
  return new _VariableNode["default"]("Math.round(require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toHsl().h)"));
};

exports.hue = hue;

var saturation = function saturation(color) {
  return new _VariableNode["default"]("Math.round(require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toHsl().s * 100) + \"%\""));
};

exports.saturation = saturation;

var lightness = function lightness(color) {
  return new _VariableNode["default"]("Math.round(require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toHsl().l * 100) + \"%\""));
};

exports.lightness = lightness;

var hsvhue = function hsvhue(color) {
  return new _VariableNode["default"]("Math.round(require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toHsv().h)"));
};

exports.hsvhue = hsvhue;

var hsvsaturation = function hsvsaturation(color) {
  return new _VariableNode["default"]("Math.round(require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toHsv().s * 100) + \"%\""));
};

exports.hsvsaturation = hsvsaturation;

var hsvvalue = function hsvvalue(color) {
  return new _VariableNode["default"]("Math.round(require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toHsv().v * 100) + \"%\""));
};

exports.hsvvalue = hsvvalue;

var red = function red(color) {
  return new _VariableNode["default"]("require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toRgb().r"));
};

exports.red = red;

var green = function green(color) {
  return new _VariableNode["default"]("require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toRgb().g"));
};

exports.green = green;

var blue = function blue(color) {
  return new _VariableNode["default"]("require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toRgb().b"));
};

exports.blue = blue;

var alpha = function alpha(color) {
  return new _VariableNode["default"]("Math.round(require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toRgb().a * 100) / 100"));
};

exports.alpha = alpha;

var luma = function luma(color) {
  return new _VariableNode["default"]("require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").getLuminance() * 100"));
};

exports.luma = luma;

var luminance = function luminance(color) {
  var compute = function compute(rgb) {
    return (0.2126 * rgb.r / 255 + 0.7152 * rgb.g / 255 + 0.0722 * rgb.b / 255) * rgb.a;
  };

  return new _VariableNode["default"]("(".concat(compute, ")(require(\"tinycolor2\")(").concat((0, _convert.convertNode)(color), ").toRgb()) * 100"));
};

exports.luminance = luminance;

var saturate = function saturate(color, amount, method) {
  return new _VariableNode["default"]("require('tinycolor2')(".concat((0, _convert.convertNode)(color), ").saturate(parseFloat(").concat((0, _convert.convertNode)(amount), ")).toRgbString()"));
};

exports.saturate = saturate;

var desaturate = function desaturate(color, amount, method) {
  return new _VariableNode["default"]("require('tinycolor2')(".concat((0, _convert.convertNode)(color), ").desaturate(parseFloat(").concat((0, _convert.convertNode)(amount), ")).toRgbString()"));
};

exports.desaturate = desaturate;

var lighten = function lighten(color, amount, method) {
  return new _VariableNode["default"]("require('tinycolor2')(".concat((0, _convert.convertNode)(color), ").lighten(parseFloat(").concat((0, _convert.convertNode)(amount), ")).toRgbString()"));
};

exports.lighten = lighten;

var darken = function darken(color, amount, method) {
  return new _VariableNode["default"]("require('tinycolor2')(".concat((0, _convert.convertNode)(color), ").darken(parseFloat(").concat((0, _convert.convertNode)(amount), ")).toRgbString()"));
};

exports.darken = darken;

var fadein = function fadein(color, amount, method) {
  var compute = function compute(color, amount, method) {
    var alpha;

    if (typeof method !== 'undefined' && method === 'relative') {
      alpha = color.getAlpha() + color.getAlpha() * amount / 100;
    } else {
      alpha = color.getAlpha() + amount / 100;
    }

    color.setAlpha(Math.min(1, Math.max(0, alpha)));
    return color.toRgbString();
  };

  return new _VariableNode["default"]("(".concat(compute, ")(require(\"tinycolor2\")(").concat((0, _convert.convertNode)(color), "), parseFloat(").concat((0, _convert.convertNode)(amount), "), ").concat((0, _convert.convertNode)(method), ")"));
};

exports.fadein = fadein;

var fadeout = function fadeout(color, amount, method) {
  var compute = function compute(color, amount, method) {
    var alpha;

    if (typeof method !== 'undefined' && method === 'relative') {
      alpha = color.getAlpha() - color.getAlpha() * amount / 100;
    } else {
      alpha = color.getAlpha() - amount / 100;
    }

    color.setAlpha(Math.min(1, Math.max(0, alpha)));
    return color.toRgbString();
  };

  return new _VariableNode["default"]("(".concat(compute, ")(require(\"tinycolor2\")(").concat((0, _convert.convertNode)(color), "), parseFloat(").concat((0, _convert.convertNode)(amount), "), ").concat((0, _convert.convertNode)(method), ")"));
};

exports.fadeout = fadeout;

var fade = function fade(color, amount) {
  return new _VariableNode["default"]("require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").setAlpha(parseFloat(").concat((0, _convert.convertNode)(amount), ") / 100).toRgbString()"));
};

exports.fade = fade;

var spin = function spin(color, amount) {
  return new _VariableNode["default"]("require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").spin(parseFloat(").concat((0, _convert.convertNode)(amount), ")).toRgbString()"));
};

exports.spin = spin;

var mix = function mix(color1, color2, weight) {
  return new _VariableNode["default"]("require(\"tinycolor2\").mix(".concat((0, _convert.convertNode)(color1), ", ").concat((0, _convert.convertNode)(color2), ", parseFloat(").concat((0, _convert.convertNode)(weight), ")).toRgbString()"));
};

exports.mix = mix;

var greyscale = function greyscale(color) {
  return desaturate(color, 100);
};

exports.greyscale = greyscale;

var contrast = function contrast(color, dark, light, threshold) {
  var compute = function compute(t, color, dark, light, threshold) {
    if (typeof light === 'undefined') {
      light = "#FFF";
    }

    if (typeof dark === 'undefined') {
      dark = "#000";
    } // Figure out which is actually light and dark:


    if (t(dark).getLuminance() > t(light).getLuminance()) {
      var _t = light;
      light = dark;
      dark = _t;
    }

    if (typeof threshold === 'undefined') {
      threshold = 0.43;
    } else {
      threshold = parseFloat(threshold) / 100;
    }

    if (t(color).getLuminance() < threshold) {
      return light;
    } else {
      return dark;
    }
  };

  return new _VariableNode["default"]("(".concat(compute, ")(require(\"tinycolor2\"), ").concat((0, _convert.convertNode)(color), ", ").concat((0, _convert.convertNode)(dark), ", ").concat((0, _convert.convertNode)(light), ", ").concat((0, _convert.convertNode)(threshold), ")"));
};

exports.contrast = contrast;

var argb = function argb(color) {
  return new _VariableNode["default"]("require(\"tinycolor2\")(".concat((0, _convert.convertNode)(color), ").toHex8String().replace(/#(\\w{6})(\\w{2})/, \"#$2$1\")"));
}; // color


exports.argb = argb;

var tint = function tint(color, weight) {
  return mix("#ffffff", color, weight);
};

exports.tint = tint;

var shade = function shade(color, weight) {
  return mix("#000000", color, weight);
};

exports.shade = shade;