"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.round = exports.acos = exports.asin = exports.atan = exports.cos = exports.sin = exports.tan = exports.abs = exports.sqrt = exports.floor = exports.ceil = void 0;

var _VariableNode = _interopRequireDefault(require("../tree/VariableNode"));

var _convert = require("../tree/convert");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var apply = function apply(fn, n) {
  return fn(parseFloat(n)) + ('' + n).replace(/[\d.-]*/, "");
};

var applyAngle = function applyAngle(fn, n) {
  var funit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "";
  var angle = {
    rad: 1,
    deg: Math.PI / 180,
    grad: Math.PI / 200,
    turn: Math.PI * 2
  };
  var unit = ('' + n).replace(/[\d.-]*/, "") || "rad";
  return fn(parseFloat(n) * angle[unit]) + funit;
};

var ceil = function ceil(value) {
  return new _VariableNode["default"]("(".concat(apply, ")(Math.ceil, ").concat((0, _convert.convertNode)(value), ")"));
};

exports.ceil = ceil;

var floor = function floor(value) {
  return new _VariableNode["default"]("(".concat(apply, ")(Math.floor, ").concat((0, _convert.convertNode)(value), ")"));
};

exports.floor = floor;

var sqrt = function sqrt(value) {
  return new _VariableNode["default"]("(".concat(apply, ")(Math.sqrt, ").concat((0, _convert.convertNode)(value), ")"));
};

exports.sqrt = sqrt;

var abs = function abs(value) {
  return new _VariableNode["default"]("(".concat(apply, ")(Math.abs, ").concat((0, _convert.convertNode)(value), ")"));
};

exports.abs = abs;

var tan = function tan(value) {
  return new _VariableNode["default"]("(".concat(applyAngle, ")(Math.tan, ").concat((0, _convert.convertNode)(value), ")"));
};

exports.tan = tan;

var sin = function sin(value) {
  return new _VariableNode["default"]("(".concat(applyAngle, ")(Math.sin, ").concat((0, _convert.convertNode)(value), ")"));
};

exports.sin = sin;

var cos = function cos(value) {
  return new _VariableNode["default"]("(".concat(applyAngle, ")(Math.cos, ").concat((0, _convert.convertNode)(value), ")"));
};

exports.cos = cos;

var atan = function atan(value) {
  return new _VariableNode["default"]("(".concat(applyAngle, ")(Math.atan, ").concat((0, _convert.convertNode)(value), ", \"rad\")"));
};

exports.atan = atan;

var asin = function asin(value) {
  return new _VariableNode["default"]("(".concat(applyAngle, ")(Math.asin, ").concat((0, _convert.convertNode)(value), ", \"rad\")"));
};

exports.asin = asin;

var acos = function acos(value) {
  return new _VariableNode["default"]("(".concat(applyAngle, ")(Math.acos, ").concat((0, _convert.convertNode)(value), ", \"rad\")"));
};

exports.acos = acos;

var round = function round(value) {
  var fraction = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return new _VariableNode["default"]("(".concat(apply, ")(num => num.toFixed(").concat(fraction.value, "), ").concat((0, _convert.convertNode)(value), ")"));
};

exports.round = round;