"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _path = _interopRequireDefault(require("path"));

var _findBabelConfig = _interopRequireDefault(require("find-babel-config"));

var _less = _interopRequireDefault(require("less/lib/less/"));

var _fileManager = _interopRequireDefault(require("less/lib/less-node/file-manager"));

var _utils2 = _interopRequireDefault(require("less/lib/less/utils"));

var _Variable = _interopRequireDefault(require("../tree/Variable"));

var _Condition = _interopRequireDefault(require("../tree/Condition"));

var _Negative = _interopRequireDefault(require("../tree/Negative"));

var _Dimension = _interopRequireDefault(require("../tree/Dimension"));

var _JavaScript = _interopRequireDefault(require("../tree/JavaScript"));

var functions = _interopRequireWildcard(require("../functions"));

var _utils3 = require("./utils");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var transpile = function transpile(less, source, filename) {
  var opts = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var banner = "";

  if (opts["import"]) {
    banner += "@import (reference) \"".concat(opts["import"], "\";");
  }

  var paths = [_path["default"].dirname(filename)]; // Add the last semi-colon if needed

  source = source.trim();

  if (!source.endsWith(";")) {
    source += ";";
  }

  switch (opts.cwd) {
    case "babelrc":
      paths.push(_path["default"].dirname(_findBabelConfig["default"].sync(filename).file));
      break;

    default:
    case "cwd":
      paths.push(process.cwd());
      break;
  }

  var parseOpts = Object.assign({}, {
    math: 0,
    paths: paths,
    banner: banner,
    syncImport: true
  }, opts.lessOptions);
  var root, imports, options;
  less.parse(source, parseOpts, function (e, _root, _imports, _options) {
    root = _root;
    imports = _imports;
    options = _options;
  });

  if (!root) {
    console.error("Failed to parse", source);
    return source;
  } // Disables the validation: Properties must be inside selector blocks. They cannot be in the root


  root.firstRoot = false;
  var parseTree = new less.ParseTree(root, imports);

  var _parseTree$toCSS = parseTree.toCSS(options),
      css = _parseTree$toCSS.css;

  return css;
};

var _default = function _default(source, filename, opts) {
  var fileManager = new _fileManager["default"](); // Adding support for scoped packages by removing the leading '~'

  fileManager.loadFile = function (filename, currentDirectory, options, environment, callback) {
    return _fileManager["default"].prototype.loadFile.call(this, filename.replace(/^~/, ""), currentDirectory, options, environment, callback);
  };

  var less = new _less["default"](undefined, [fileManager]);
  var oldGenCSS = less.tree.Expression.prototype.genCSS;
  var oldEval = less.tree.Anonymous.prototype.eval;
  var oldTree = Object.assign({}, less.tree);
  var oldFunctionRegistry = Object.assign({}, less.functions.functionRegistry._data);
  var oldJoinSelector = less.tree.Ruleset.prototype.joinSelector;

  try {
    Object.defineProperty(less.tree.Node.prototype, 'parse', (0, _utils3.nodeParse)(less));
    less.tree.Variable = _Variable["default"];
    less.tree.Condition = _Condition["default"];
    less.tree.Negative = _Negative["default"];
    less.tree.Dimension = _Dimension["default"];
    less.tree.JavaScript = _JavaScript["default"];
    less.tree.Expression.prototype.genCSS = (0, _utils3.genCSS)(less);
    less.tree.Anonymous.prototype.eval = (0, _utils3.anonymousEval)(less); // Changes the function joinSelector to allow & selectors in the root element. Useful for overriding styles with higher specificity.

    var _less$tree = less.tree,
        Paren = _less$tree.Paren,
        Selector = _less$tree.Selector,
        Element = _less$tree.Element;
    var joinSelector = less.tree.Ruleset.prototype.joinSelector.toString().replace("if (el.value !== '&') {", "if (el.value !== '&' || context.length === 0) {");
    var utils = _utils2["default"];
    eval("less.tree.Ruleset.prototype.joinSelector = ".concat(joinSelector));
    less.functions.functionRegistry.addMultiple(functions);

    less.PluginLoader = function PluginLoader() {
      _classCallCheck(this, PluginLoader);
    };

    return transpile(less, source, filename, opts);
  } finally {
    delete less.tree.Node.prototype.parse;
    Object.assign(less.tree, oldTree);
    less.tree.Expression.prototype.genCSS = oldGenCSS;
    less.tree.Anonymous.prototype.eval = oldEval;
    less.tree.Ruleset.prototype.joinSelector = oldJoinSelector;
    less.functions.functionRegistry._data = oldFunctionRegistry;
  }
};

exports["default"] = _default;