"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.nodeParse = exports.anonymousEval = exports.genCSS = void 0;

var _VariableNode = _interopRequireDefault(require("../tree/VariableNode"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var consumeBrackets = function consumeBrackets(parserInput) {
  var inComment = false;
  var blockDepth = 0;
  var blockStack = [];
  var input = parserInput.getInput();
  var length = input.length;
  var lastPos = parserInput.i;
  var i = parserInput.i;
  var prevChar, nextChar;

  do {
    prevChar, nextChar = input.charAt(i);

    if (blockDepth === 0 && prevChar === "}") {
      return input.substr(lastPos, i - lastPos);
    } else {
      if (inComment) {
        if (nextChar === '*' && input.charAt(i + 1) === '/') {
          i++;
          blockDepth--;
          inComment = false;
        }

        i++;
        continue;
      }

      switch (nextChar) {
        case '/':
          if (input.charAt(i + 1) === '*') {
            i++;
            inComment = true;
            blockDepth++;
          }

          break;

        case "'":
        case '"':
          var quote = parserInput.$quoted(i);

          if (quote) {
            i += quote[1].length - 1;
          }

          break;

        case '{':
          blockStack.push('}');
          blockDepth++;
          break;

        case '(':
          blockStack.push(')');
          blockDepth++;
          break;

        case '[':
          blockStack.push(']');
          blockDepth++;
          break;

        case '}':
        case ')':
        case ']':
          var expected = blockStack.pop();

          if (nextChar === expected) {
            blockDepth--;
          } else {
            return expected;
          }

      }

      i++;
    }

    prevChar = nextChar;
  } while (i <= length);
};

var genCSS = function genCSS(less) {
  return function (context, output) {
    var cssFragment = context && context.cssFragment && this.value.some(function (v) {
      return v instanceof _VariableNode["default"];
    });
    if (cssFragment) output.add('`');

    for (var i = 0; i < this.value.length; i++) {
      var value = this.value[i];

      if (cssFragment && value instanceof _VariableNode["default"]) {
        output.add("${".concat(value.toCSS(), "}"));
      } else {
        value.genCSS(context, output);
      }

      if (!(this.noSpacing || value.noSpacing) && i + 1 < this.value.length) {
        output.add(' ');
      }
    }

    if (cssFragment) output.add('`');
  };
};

exports.genCSS = genCSS;

var anonymousEval = function anonymousEval(less) {
  return function () {
    var anonymous = new less.tree.Anonymous(this.value, this._index, this._fileInfo, this.mapLines, this.rulesetLike, this.visibilityInfo());
    anonymous.noSpacing = this.noSpacing;
    return anonymous;
  };
};

exports.anonymousEval = anonymousEval;
var lastSelf;

var nodeParse = function nodeParse(less) {
  return {
    get: function get() {
      return lastSelf;
    },
    set: function set(self) {
      lastSelf = self;

      var $re = function $re(parserInput, tok) {
        parserInput.save();
        var ret = parserInput.$re(tok);
        parserInput.restore();
        return ret;
      };

      var parseJS = function parseJS(orig, treeConstructor, isDeclaration) {
        var val = orig();
        if (val) return val;
        var name;
        self.parserInput.save();

        if ($re(self.parserInput, /^\.?\$/) && (name = consumeBrackets(self.parserInput))) {
          // Makes the assumption that all nested code blocks have css in them, or ends with a ;.
          if (isDeclaration && !name.includes("css")) {
            self.parserInput.save();
            self.parserInput.$str(name);

            if (self.parserInput.currentChar() !== ';') {
              self.parserInput.restore();
              return;
            }
          }

          self.parserInput.$str(name);
          var anonymous = new less.tree.Anonymous(name, self.parserInput.i, self.fileInfo);
          anonymous.noSpacing = self.parserInput.prevChar() !== " ";

          if (treeConstructor) {
            return treeConstructor(anonymous);
          } else {
            return anonymous;
          }
        }

        self.parserInput.restore();
      };

      self.parsers.declaration = parseJS.bind(null, self.parsers.declaration.bind(self.parsers), undefined, true);
      self.parsers.entities.variable = parseJS.bind(null, self.parsers.entities.variable.bind(self.parsers));
      self.parsers.element = parseJS.bind(null, self.parsers.element.bind(self.parsers), function (anonymous) {
        var c = self.parsers.combinator();
        return new less.tree.Element(c, anonymous, true, self.parserInput.i, self.fileInfo);
      });
    },
    configurable: true
  };
};

exports.nodeParse = nodeParse;